
	/* some miscellaneous functions used by several programs */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "pf.h"
#include "misc.h"
#include "isvalid.h"

#ifdef VERBOSE
extern char *progname;
#endif

static int days[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

static int is_leap_year (int year);

	/* translate the pf_string (i.e. one that still has single-quote marks
	   around it) of form "DD/MM/YYYY" into numerical values for
	   the day, month and year. returns -1 if there's a problem, or 0
	   if all went well. */ 

int 
read_date(val, day, month, year)
char *val;
int *day, *month, *year;
{
	char str[PF_LEN + 1];

	if (pf_valtostr(val, str) < 0)
		return(-1);
	if (is_date(str) != 0) 
		return(-1);
	if (sscanf(str, "%d/%d/%d", day, month, year) != 3) 
		return(-1);
	return(0);
}

	/* return the number of days between the given date and Jan 0, 1950 */

int 
since_1950(day, month, year)
int day, month, year;
{
	int i, sum;

	sum = 0;
	for (i = 1950; i < year; i++) {
		sum += 365;
		/* check for leap year ... */
		if (is_leap_year(i)) {
			sum++;
		}
	}
	for (i = 1; i < month; i++) {
		sum += days[i - 1];
		if ((i == 2) && (is_leap_year(year))) {
			sum++;
		}
	}
	sum += day;
	return(sum);
}


	/**********************************************************************
	 * Given an integer year value (like 1954 or 2001), decide it it's
	 * a leap year.  
	 *
	 * RETURNS:
	 *    1            if it is a leap year
	 *    0            if not 
	 */

static int
is_leap_year
	(
	int year
	)
{
	if (year % 4 != 0) {
		return(0);
	}

	/* if we get here, we must have a year divisible by 4 */
	/* if it's not divisible by 100, then it's definitely a leap year */
	if (year % 100 != 0) {
		return(1);
	}

	/* if we get here, we have a century year: 1900, 2000, 2100, etc. */
	/* If the century is divisible by 400, it's a leap year; otherwise, not */
	if (year % 400 == 0) {
		return(1);
	} else {
		return(0);
	}

	/* we should never reach here */
	fprintf(stderr, "is_leap_year: fell to bottom of function for year %d?!\n",
			year);
	exit(1);
}

	/* convert a time, which was expressed as a string of form HH:MM:SS, 
	   to a single quantity which is simply the fractional hours
	   seconds since 00:00:00.  If there is some problem, return -1. 
	   Otherwise, return 0. */

int
read_timestr(str, fhour)
char *str;
double *fhour;
{
	int hour, minute, second;

	if (sscanf(str, "%2d:%2d:%2d", &hour, &minute, &second) != 3)
		return(-1);
	if ((hour < 0) || (hour > 23) || (minute < 0) || (minute > 59) ||
	    (second < 0) || (second > 59))
		return(-1);

	*fhour = (double)hour + (double)minute/60.0 + (double)second/3600.0;
	return(0);
}

	/* convert a time from fractional hours to a string of the form
       HH:MM:SS, exactly 8 characters long, plus a null character at the
	   end. return 0. */

int
write_timestr(fhour, str)
double fhour;
char *str;
{
	int hour, minute, second;

	hour = fhour;		/* simple truncation */
	minute = (fhour - hour)*60.0;	
	second = ((fhour - hour) - minute/60.0)*3600.0;
	sprintf(str, "%02d:%02d:%02d", hour, minute, second);
	return(0);
}




	/*	writes error message to stderr then exits if 'err' is non-zero */

void 
error(err, message)
int err;
char *message;
{

	fprintf(stderr,"\n%s\n",message);
	if (err)
		exit(err);
}


/*	
	finds the right hand value of the equality
	returns a pointer to first character after equal sign
	or NULL if not successful
*/


char *
find(lval, rval)
char *lval, *rval;
{
	int len;
	
	len = strlen(lval);
	if ((strncmp(lval, rval, len) == 0) && (rval[len] == '='))
		return(rval + len + 1);
	return(NULL);	
}

int 
keyword(lval, rval)		/* returns non-zero when 'lval' is found in 'rval' */
char *lval, *rval;
{
	return(!strcmp(lval, rval));
}


	/* turn the given string into the lower-case version of itself */

void
str_to_lower(str)
char *str;
{
	static int diff = 'A' - 'a';
	char *p;

	for (p = str; *p != '\0'; p++) {
		if (isupper(*p))
			*p -= diff;
	}
}

	/* turn the given string into the upper-case version of itself */

void
str_to_upper(str)
char *str;
{
	static int diff = 'A' - 'a';
	char *p;

	for (p = str; *p != '\0'; p++) {
		if (isupper(*p))
			*p += diff;
	}
}
